import { equals } from 'ramda';
import * as React from 'react';
import { Dimensions, findNodeHandle, Platform, StyleSheet, UIManager, } from 'react-native';
import { FlatList, LongPressGestureHandler, PanGestureHandler, State as GestureState, TapGestureHandler, } from 'react-native-gesture-handler';
import Animated, { Easing } from 'react-native-reanimated';
import { getStatusBarHeight } from 'react-native-status-bar-height';
const isIOS = Platform.OS === 'ios';
const { height: winHeight } = Dimensions.get('window');
const iPhoneXFullScreenHeight = 812;
const iPhoneXMaxFullScreenHeight = 896;
const isIPhoneX = isIOS &&
    (winHeight === iPhoneXFullScreenHeight ||
        winHeight === iPhoneXMaxFullScreenHeight);
const iPhoneXBottomInset = 34;
const statusBarHeight = !isIOS ? getStatusBarHeight() : 0;
const { event, block, eq, neq, set, Value, call, cond, add, sub, lessThan, lessOrEq, greaterThan, greaterOrEq, and, or, divide, multiply, onChange, round, Clock, timing, startClock, stopClock, clockRunning, interpolate, } = Animated;
const AnimatedFlatList = Animated.createAnimatedComponent(FlatList);
const styles = StyleSheet.create({
    flexContainer: {
        flex: 1,
        width: '100%',
    },
    flexContainerOverflow: {
        flex: 1,
        width: '100%',
        overflow: 'hidden',
    },
});
const getIsDataMultipleLists = (data) => Array.isArray(data[0]);
const setUpInitialData = (data) => {
    const isDataMultipleLists = getIsDataMultipleLists(data);
    if (isDataMultipleLists) {
        const flatData = data.reduce((acc, arr, arrayIndex) => {
            return [
                ...acc,
                ...arr.map((item, itemIndex) => ({
                    ...item,
                    arrayIndex,
                    draggableId: acc.length + itemIndex,
                    draggableIdValue: new Value(acc.length + itemIndex),
                    itemTranslateY: new Value(0),
                    itemKey: acc.length + itemIndex,
                })),
            ];
        }, []);
        return flatData;
    }
    const dataWithId = data.map((item, index) => {
        return {
            ...item,
            draggableId: index,
            draggableIdValue: new Value(index),
            arrayIndex: 0,
            itemTranslateY: new Value(0),
            itemKey: index,
        };
    });
    return dataWithId;
};
const mapItemBoundaries = (itemHeights, itemOffsets, firstItemIds) => (item, index) => {
    const { arrayIndex } = item;
    const itemHeight = itemHeights[arrayIndex];
    const offset = itemOffsets[arrayIndex];
    const firstItemIndex = firstItemIds[arrayIndex];
    const offsetIndex = index - firstItemIndex;
    const top = offsetIndex * itemHeight + offset;
    const bottom = top + itemHeight;
    return { top, bottom };
};
export class SortableMultilist extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            containerHeight: 0,
            containerWidth: 0,
            containerOffsetLeft: 0,
            itemHeights: [0],
            itemBoundaries: [],
            scrollSpeed: 0,
            dataWithId: setUpInitialData(this.props.data),
            isDataMultipleLists: getIsDataMultipleLists(this.props.data),
            largestItemHeight: 0,
            initialSetupFinished: 0,
        };
        this.mapFirstItemsIds = () => {
            if (!this.state.isDataMultipleLists) {
                return [0, 1];
            }
            const { data } = this.props;
            return data.reduce((final, array, index, dataArray) => {
                if (index === 0) {
                    return [0];
                }
                const previousIndex = index - 1;
                return [
                    ...final,
                    final[previousIndex] + dataArray[previousIndex].length,
                ];
            }, []);
        };
        this.mapLastItemIds = () => {
            const { data } = this.props;
            if (!this.state.isDataMultipleLists) {
                return [data.length - 1, 0];
            }
            return data.reduce((final, array, index, dataArray) => {
                if (index === 0) {
                    return [array.length - 1];
                }
                return [...final, array.length + final[index - 1]];
            }, []);
        };
        this.mapFirstItemRefs = () => {
            if (!this.state.isDataMultipleLists) {
                return [React.createRef()];
            }
            return this.props.data.map(() => React.createRef());
        };
        this.mapItemArraysIds = (firstItemIds) => {
            const arrayIds = firstItemIds.map((item, index) => index);
            return arrayIds;
        };
        this.mapActiveHoverZones = (firstItemIds) => {
            const defaultZone = { top: 0, bottom: 0 };
            const activeHoverZones = this.firstItemIds.map(() => defaultZone);
            return activeHoverZones;
        };
        this.mapHeaderRefs = () => {
            if (!this.props.renderHeader) {
                return null;
            }
            if (typeof this.props.renderHeader === 'function') {
                return [React.createRef()];
            }
            const refs = this.props.renderHeader.map((renderFn) => {
                if (typeof renderFn === 'function') {
                    return React.createRef();
                }
                return null;
            });
            return refs;
        };
        this.resetClassVariables = () => {
            this.scrollEnabled = true;
            this.hasAutoScrollStartedValue.setValue(0);
            this.isAutoScrollDownValue.setValue(0);
            this.isAutoScrollUpValue.setValue(0);
            this.activeItemIndex = -1;
            this.activeItemIndexValue.setValue(-1);
            this.activeItemArrayIndexValue.setValue(-1);
            this.activeHoverIndex = -1;
            this.activeHoverIndexValue.setValue(-1);
            this.activeHoverArrayIndexValue.setValue(-1);
            this.activeHoverArrayIndex = -1;
            this.activeHoverOutOfBoundsValue.setValue(0);
            this.activeHoverZoneValue.setValue(-1);
            this.touchOutOfBoundsValue.setValue(0);
            this.touchOutOfBoundsTopValue.setValue(0);
            this.touchOutOfBoundsBottomValue.setValue(0);
            this.isSavingValue.setValue(0);
        };
        this.getListForSaving = () => {
            const { data } = this.props;
            const { isDataMultipleLists } = this.state;
            const dataToSave = isDataMultipleLists
                ? data.map((list, index) => {
                    const updatedList = this.dataWithProps
                        .filter((item) => item.arrayIndex === index)
                        .map((item) => {
                        const { arrayIndex, draggableId, itemKey, itemTranslateY, draggableIdValue, ...cleansedItem } = item;
                        return cleansedItem;
                    });
                    return updatedList;
                })
                : this.dataWithProps.reduce((final, item) => {
                    const { arrayIndex, draggableId, itemKey, itemTranslateY, draggableIdValue, ...cleansedItem } = item;
                    return [...final, cleansedItem];
                }, []);
            return dataToSave;
        };
        this.updateListOnDragEnd = () => {
            const { data, updateListOnDragEnd, updateListDebounceDuration, disableUpdateListDebounce, disableAutoUpdate, } = this.props;
            const dataToSave = this.getListForSaving();
            this.dataToSave = dataToSave;
            if (equals(data, this.dataToSave) ||
                disableAutoUpdate ||
                !updateListOnDragEnd) {
                return;
            }
            if (disableUpdateListDebounce) {
                updateListOnDragEnd(dataToSave);
                return;
            }
            clearTimeout(this.updateListTimeout);
            this.updateListTimeout = setTimeout(() => {
                updateListOnDragEnd(dataToSave);
            }, updateListDebounceDuration);
        };
        this.flatListRef = React.createRef();
        this.tapRef = React.createRef();
        this.longPressRef = React.createRef();
        this.panGestureRef = React.createRef();
        this.firstItemRefs = this.mapFirstItemRefs();
        this.headerRefs = this.mapHeaderRefs();
        this.firstItemIds = this.mapFirstItemsIds();
        this.lastItemIds = this.mapLastItemIds();
        this.itemArrayIds = this.mapItemArraysIds(this.firstItemIds);
        this.itemOffsets = this.firstItemIds.map(() => 0);
        this.activeHoverZones = this.mapActiveHoverZones(this.firstItemIds);
        this.updateListTimeout = null;
        this.dataWithProps = this.state.dataWithId;
        this.dataToSave = this.props.data;
        this.activeItemIndex = -1;
        this.activeItemArrayIndex = -1;
        this.activeHoverIndex = -1;
        this.activeHoverArrayIndex = -1;
        this.scrollEnabled = true;
        this.initialSetUpFinishedValue = new Value(0);
        this.isSavingValue = new Value(0);
        this.longPressStateValue = new Value(0);
        this.tapStateValue = new Value(0);
        this.gestureStateValue = new Value(0);
        this.gestureOldStateValue = new Value(0);
        this.containerHeightValue = new Value(0);
        this.containerOffsetTopValue = new Value(0);
        this.containerLowerBoundValue = new Value(0);
        this.containerUpperBoundValue = new Value(0);
        this.contentTotalHeightValue = new Value(0);
        this.hiddenContentHeightValue = new Value(0);
        this.pointerY = new Value(0);
        this.pointerX = new Value(0);
        this.scrollTopValue = new Value(0);
        this.scrollTopLowerBoundValue = new Value(0);
        this.autoScrollTargetValue = new Value(0);
        this.hasAutoScrollStartedValue = new Value(0);
        this.hasScrollTopMatchedTargetValue = new Value(0);
        this.isAutoScrollDownValue = new Value(0);
        this.isAutoScrollUpValue = new Value(0);
        this.activeItemIndexValue = new Value(-1);
        this.activeHoverIndexValue = new Value(-1);
        this.activeItemArrayIndexValue = new Value(-1);
        this.activeHoverArrayIndexValue = new Value(-1);
        this.lastItemIndexValue = new Value(0);
        this.touchOutOfBoundsValue = new Value(0);
        this.touchOutOfBoundsTopValue = new Value(0);
        this.touchOutOfBoundsBottomValue = new Value(0);
        this.activeHoverOutOfBoundsValue = new Value(0);
        this.activeHoverZoneValue = new Value(-1);
        this.itemHeightValues = this.state.isDataMultipleLists
            ? this.props.data.map(() => new Value(0))
            : [new Value(0), new Value(1)];
        this.getReorderedData = () => {
            const { dataWithProps } = this;
            if (this.activeHoverIndex === -1 || this.activeHoverArrayIndex === -1) {
                return dataWithProps;
            }
            const { arrayIndex: activeItemArrayIndex } = dataWithProps[this.activeItemIndex];
            if (activeItemArrayIndex !== this.activeHoverArrayIndex) {
                return dataWithProps;
            }
            const dataWithoutActive = dataWithProps.filter((item) => item.draggableId !== this.activeItemIndex);
            const head = this.activeHoverIndex === 0
                ? []
                : dataWithoutActive.slice(0, this.activeHoverIndex);
            const tail = dataWithoutActive.slice(this.activeHoverIndex);
            const updatedDataList = [
                ...head,
                dataWithProps[this.activeItemIndex],
                ...tail,
            ];
            const remappedIds = updatedDataList.map((item, index) => ({
                ...item,
                draggableId: index,
                draggableIdValue: new Value(index),
            }));
            return remappedIds;
        };
        this.handleFlatListLayout = async ({ nativeEvent }) => {
            const { layout: { height, width }, } = nativeEvent;
            const { itemHeights, isDataMultipleLists } = this.state;
            const flatListOffset = await new Promise(async (resolve, reject) => {
                await UIManager.measure(findNodeHandle(this.flatListRef.current), async (x, y, w, h, px, py) => {
                    resolve([px, py]);
                });
            });
            const [flatListOffsetLeft, flatListOffsetTop] = flatListOffset;
            if (this.state.initialSetupFinished === 0) {
                const isReady = await new Promise((resolve) => {
                    const lastIndex = this.firstItemRefs.length - 1;
                    if (this.firstItemRefs[lastIndex].current) {
                        resolve(true);
                        return;
                    }
                    resolve(false);
                });
                if (!isReady) {
                    setTimeout(() => this.handleFlatListLayout({ nativeEvent }), 50);
                    return;
                }
            }
            const itemHeightResolves = this.firstItemRefs.map((ref) => new Promise((resolve, reject) => {
                UIManager.measure(findNodeHandle(ref.current), (x, y, w, h) => resolve(h));
            }));
            const updatedItemHeights = await Promise.all(itemHeightResolves);
            const headerHeightResolves = this.headerRefs
                ? this.headerRefs.map((header) => new Promise((resolve, reject) => {
                    if (header === null || header.current === null) {
                        return resolve(0);
                    }
                    UIManager.measure(findNodeHandle(header.current), (x, y, w, h) => resolve(h || 0));
                }))
                : null;
            const updatedHeaderHeights = headerHeightResolves
                ? await Promise.all(headerHeightResolves)
                : [0];
            if (!equals(updatedItemHeights, itemHeights)) {
                const largestItemHeight = updatedItemHeights.reduce((final, current) => (current > final ? current : final), 0);
                const scrollSpeed = largestItemHeight;
                const actualSpeed = Math.round(isIOS ? scrollSpeed : scrollSpeed * 0.1);
                const itemBoundaryOffset = largestItemHeight;
                const requiresOffset = isIPhoneX &&
                    [iPhoneXFullScreenHeight, iPhoneXMaxFullScreenHeight].includes(height);
                const bottomInset = requiresOffset ? iPhoneXBottomInset : 0;
                const topInset = requiresOffset ? 0 : flatListOffsetTop + statusBarHeight;
                this.containerHeightValue.setValue(height);
                this.containerLowerBoundValue.setValue(topInset + height - bottomInset - itemBoundaryOffset);
                this.containerUpperBoundValue.setValue(flatListOffsetTop + itemBoundaryOffset + statusBarHeight);
                this.containerOffsetTopValue.setValue(flatListOffsetTop + statusBarHeight);
                this.itemHeightValues.forEach((itemHeightValue, index) => {
                    const itemHeight = updatedItemHeights[index] || updatedItemHeights[index - 1];
                    itemHeightValue.setValue(itemHeight);
                });
                this.itemOffsets = this.firstItemIds.map((draggableId, firstItemId) => {
                    if (!this.state.isDataMultipleLists && firstItemId > 0) {
                        return 0;
                    }
                    const offset = [...Array(firstItemId)]
                        .map((ignore, innerIndex) => (this.firstItemIds[innerIndex + 1] -
                        this.firstItemIds[innerIndex]) *
                        updatedItemHeights[innerIndex])
                        .reduce((acc, current) => acc + current, 0);
                    const headerOffsets = [...Array(firstItemId + 1)]
                        .map((ignore, innerIndex) => {
                        const headerOffset = updatedHeaderHeights[innerIndex];
                        return headerOffset ? headerOffset : 0;
                    })
                        .reduce((acc, current) => acc + current, 0);
                    return offset + headerOffsets;
                });
                const boundaryMapper = mapItemBoundaries(updatedItemHeights, this.itemOffsets, this.firstItemIds);
                const updatedItemBoundaries = this.dataWithProps.map(boundaryMapper);
                const totalHeaderHeights = updatedHeaderHeights.reduce((final, current) => final + current, 0);
                const contentTotalHeight = totalHeaderHeights +
                    updatedItemHeights.reduce((final, current, index) => {
                        const { data } = this.props;
                        const dataLength = this.state.isDataMultipleLists
                            ? data[index].length
                            : data.length;
                        return final + current * dataLength;
                    }, 0);
                const updatedActiveHoverZones = this.itemArrayIds.map((arrayId) => {
                    if (!isDataMultipleLists && arrayId > 0) {
                        return {
                            top: contentTotalHeight,
                            bottom: contentTotalHeight,
                        };
                    }
                    const top = this.itemOffsets[arrayId];
                    const bottom = this.itemOffsets[arrayId + 1] - updatedHeaderHeights[arrayId + 1] ||
                        contentTotalHeight;
                    return { top, bottom };
                });
                this.activeHoverZones = updatedActiveHoverZones;
                this.contentTotalHeightValue.setValue(contentTotalHeight);
                this.hiddenContentHeightValue = sub(this.contentTotalHeightValue, this.containerHeightValue);
                this.lastItemIndexValue.setValue(this.dataWithProps.length - 1);
                this.setState({
                    containerHeight: height,
                    containerWidth: width,
                    containerOffsetLeft: flatListOffsetLeft,
                    itemHeights: updatedItemHeights,
                    itemBoundaries: updatedItemBoundaries,
                    scrollSpeed: actualSpeed,
                    largestItemHeight,
                    initialSetupFinished: 1,
                }, () => {
                    this.initialSetUpFinishedValue.setValue(1);
                });
            }
        };
        this.calculateActiveItem = (initialY, scrollTop, containerOffsetTop) => {
            const { itemBoundaries } = this.state;
            const touchY = initialY - containerOffsetTop + scrollTop;
            const currentActiveItemIndex = itemBoundaries.reduce((final, bounds, index) => {
                const { top, bottom } = bounds;
                if (touchY > top && touchY < bottom) {
                    return index;
                }
                return final;
            }, -1);
            return currentActiveItemIndex;
        };
        this.handleTapStateChange = (params) => {
            const [state, gestureState, absoluteY, absoluteX, scrollTop, containerOffsetTop,] = params;
            const { enableRightDragArea, rightDragAreaOffsets } = this.props;
            if (state === GestureState.BEGAN && enableRightDragArea) {
                const { containerWidth, containerOffsetLeft } = this.state;
                const { width = 0, marginRight = 0 } = rightDragAreaOffsets || {};
                const activeAreaCoef = 0.2;
                const activeArea = rightDragAreaOffsets
                    ? width + marginRight
                    : containerWidth * activeAreaCoef;
                const activeAreaLeftBoundary = containerOffsetLeft + containerWidth - activeArea;
                const activeAreaRightBoundary = containerOffsetLeft + containerWidth - marginRight;
                if (absoluteX > activeAreaLeftBoundary &&
                    absoluteX < activeAreaRightBoundary) {
                    const activeItemIndex = this.calculateActiveItem(absoluteY, scrollTop, containerOffsetTop);
                    if (activeItemIndex > -1) {
                        clearTimeout(this.updateListTimeout);
                        this.activeItemIndexValue.setValue(activeItemIndex);
                        this.activeItemIndex = activeItemIndex;
                        this.flatListRef.current.setNativeProps({ scrollEnabled: false });
                        this.scrollEnabled = false;
                        const { arrayIndex } = this.dataWithProps[activeItemIndex];
                        this.activeItemArrayIndexValue.setValue(arrayIndex);
                        this.activeItemArrayIndex = arrayIndex;
                    }
                }
            }
            if (state === GestureState.END && gestureState !== GestureState.ACTIVE) {
                this.flatListRef.current.setNativeProps({ scrollEnabled: true });
                this.resetClassVariables();
                const { disableUpdateListDebounce, disableAutoUpdate } = this.props;
                if (!disableAutoUpdate && !disableUpdateListDebounce) {
                    this.updateListOnDragEnd();
                }
            }
        };
        this.onTapStateChange = event([
            {
                nativeEvent: ({ state, absoluteY, absoluteX }) => cond(and(eq(this.initialSetUpFinishedValue, 1), eq(this.isSavingValue, 0)), [
                    set(this.tapStateValue, state),
                    onChange(this.tapStateValue, [
                        cond(eq(state, GestureState.BEGAN), [
                            set(this.pointerY, absoluteY),
                            set(this.pointerX, absoluteX),
                        ]),
                        call([
                            this.tapStateValue,
                            this.gestureStateValue,
                            this.pointerY,
                            this.pointerX,
                            this.scrollTopValue,
                            this.containerOffsetTopValue,
                        ], this.handleTapStateChange),
                    ]),
                ]),
            },
        ]);
        this.handleLongPressActive = (params) => {
            if (this.props.disableLongPress) {
                return;
            }
            const [touchY, scrollTop, containerOffsetTop, isSaving, alreadyActive,] = params;
            if (isSaving || alreadyActive !== -1) {
                return;
            }
            const activeItemIndex = this.calculateActiveItem(touchY, scrollTop, containerOffsetTop);
            if (activeItemIndex === -1) {
                return;
            }
            clearTimeout(this.updateListTimeout);
            this.flatListRef.current.setNativeProps({
                scrollEnabled: false,
            });
            this.scrollEnabled = false;
            this.activeItemIndex = activeItemIndex;
            this.activeItemIndexValue.setValue(activeItemIndex);
            const { arrayIndex } = this.dataWithProps[activeItemIndex];
            this.activeItemArrayIndexValue.setValue(arrayIndex);
            this.activeItemArrayIndex = arrayIndex;
        };
        this.handleLongPressCancelled = () => {
            this.flatListRef.current.setNativeProps({ scrollEnabled: true });
            this.resetClassVariables();
            const { disableAutoUpdate, disableUpdateListDebounce } = this.props;
            if (!disableAutoUpdate && !disableUpdateListDebounce) {
                this.updateListOnDragEnd();
            }
        };
        this.onLongPress = event([
            {
                nativeEvent: ({ state, absoluteY, oldState }) => block([
                    cond(and(eq(this.initialSetUpFinishedValue, 1), eq(this.isSavingValue, 0)), [
                        set(this.longPressStateValue, state),
                        onChange(this.longPressStateValue, [
                            cond(eq(this.longPressStateValue, GestureState.ACTIVE), [
                                set(this.pointerY, absoluteY),
                                call([
                                    this.pointerY,
                                    this.scrollTopValue,
                                    this.containerOffsetTopValue,
                                    this.isSavingValue,
                                    this.activeItemIndexValue,
                                ], this.handleLongPressActive),
                            ]),
                            cond(and(neq(this.gestureStateValue, GestureState.ACTIVE), and(eq(oldState, GestureState.ACTIVE), or(eq(state, GestureState.END), eq(state, GestureState.FAILED)))), call([], this.handleLongPressCancelled)),
                        ]),
                    ]),
                ]),
            },
        ]);
        this.setScrollLowerBound = (contentWidth, contentHeight) => {
            const scrollDiff = contentHeight - this.state.containerHeight + this.state.largestItemHeight;
            this.scrollTopLowerBoundValue.setValue(scrollDiff);
        };
        this.autoScrollComplete = (params) => {
            if (this.autoScrollResolve) {
                this.autoScrollResolve(params);
                this.autoScrollResolve = null;
            }
        };
        this.callAutoScrollComplete = call([
            this.scrollTopValue,
            this.scrollTopLowerBoundValue,
            this.isAutoScrollDownValue,
            this.isAutoScrollUpValue,
            this.touchOutOfBoundsValue,
        ], this.autoScrollComplete);
        this.onScroll = event([
            {
                nativeEvent: ({ contentOffset }) => block([
                    set(this.scrollTopValue, contentOffset.y),
                    cond(and(eq(this.isAutoScrollDownValue, 1), greaterOrEq(round(this.scrollTopValue), round(this.autoScrollTargetValue))), [
                        set(this.hasScrollTopMatchedTargetValue, 1),
                        this.callAutoScrollComplete,
                    ]),
                    cond(and(eq(this.isAutoScrollUpValue, 1), lessOrEq(round(this.scrollTopValue), round(this.autoScrollTargetValue))), [
                        set(this.hasScrollTopMatchedTargetValue, 1),
                        this.callAutoScrollComplete,
                    ]),
                    cond(and(and(eq(this.isAutoScrollDownValue, 1), eq(this.hasScrollTopMatchedTargetValue, 1)), lessThan(this.scrollTopValue, this.autoScrollTargetValue)), set(this.hasScrollTopMatchedTargetValue, 0)),
                    cond(and(and(eq(this.isAutoScrollUpValue, 1), eq(this.hasScrollTopMatchedTargetValue, 1)), greaterThan(this.scrollTopValue, this.autoScrollTargetValue)), set(this.hasScrollTopMatchedTargetValue, 0)),
                ]),
            },
        ]);
        this.scrollToAsync = (scrollTarget) => new Promise((resolve, reject) => {
            this.autoScrollResolve = resolve;
            this.autoScrollTargetValue.setValue(scrollTarget);
            this.flatListRef.current._component.scrollToOffset({
                offset: scrollTarget,
            });
        });
        this.autoScroll = async (params) => {
            const [scrollTop, scrollTopEnd, isAutoScrollDownValue, isAutoScrollUpValue, isTouchOutOfBounds,] = params;
            if ((isAutoScrollDownValue && scrollTop === scrollTopEnd) ||
                (isAutoScrollUpValue && scrollTop === 0)) {
                return;
            }
            const scrollTarget = Math.round(isAutoScrollDownValue
                ? Math.min(scrollTop + this.state.scrollSpeed, scrollTopEnd)
                : Math.max(0, scrollTop - this.state.scrollSpeed));
            const nextScrollParams = await this.scrollToAsync(scrollTarget);
            if (!this.scrollEnabled && !isTouchOutOfBounds) {
                this.autoScroll(nextScrollParams);
            }
        };
        this.trackHover = (params) => {
            const [yPos, scrollTop, containerOffsetTop, activeItemArray] = params;
            const currentActiveHoverIndex = this.calculateActiveItem(yPos, scrollTop, containerOffsetTop);
            if (currentActiveHoverIndex !== this.activeHoverIndex &&
                currentActiveHoverIndex !== -1) {
                this.activeHoverIndex = currentActiveHoverIndex;
                this.activeHoverIndexValue.setValue(currentActiveHoverIndex);
                const { arrayIndex } = this.dataWithProps[currentActiveHoverIndex];
                this.activeHoverArrayIndexValue.setValue(arrayIndex);
                this.activeHoverArrayIndex = arrayIndex;
            }
            const touchY = yPos - containerOffsetTop + scrollTop;
            const activeZone = this.activeHoverZones.reduce((final, bounds, index) => {
                const { top, bottom } = bounds;
                if (touchY > top && touchY < bottom) {
                    return index;
                }
                return final;
            }, -1);
            if (activeZone === activeItemArray) {
                this.activeHoverZoneValue.setValue(activeZone);
                this.activeHoverOutOfBoundsValue.setValue(0);
                return;
            }
            if (this.activeHoverZones[activeItemArray] &&
                touchY < this.activeHoverZones[activeItemArray].top) {
                this.activeHoverZoneValue.setValue(activeItemArray - 1);
            }
            else {
                this.activeHoverZoneValue.setValue(activeItemArray + 1);
            }
            this.activeHoverOutOfBoundsValue.setValue(1);
        };
        this.updateAllItemY = (updatedDataList) => {
            this.state.dataWithId.forEach(({ draggableId: initialDraggableId, draggableIdValue, itemKey: itemKeyState, }) => {
                const { draggableId, arrayIndex, itemTranslateY, } = updatedDataList.find(({ itemKey }) => itemKey === itemKeyState);
                draggableIdValue.setValue(draggableId);
                const itemHeight = this.state.itemHeights[arrayIndex];
                const diff = draggableId - initialDraggableId;
                const newY = diff * itemHeight;
                itemTranslateY.setValue(newY);
            });
        };
        this.callAutoScroll = call([
            this.scrollTopValue,
            this.scrollTopLowerBoundValue,
            this.isAutoScrollDownValue,
            this.isAutoScrollUpValue,
            this.touchOutOfBoundsValue,
        ], this.autoScroll);
        this.handleSave = (params) => {
            const [activeItem] = params;
            if (activeItem === -1) {
                this.flatListRef.current.setNativeProps({ scrollEnabled: true });
                this.resetClassVariables();
                return;
            }
            const updatedDataList = this.getReorderedData();
            this.updateAllItemY(updatedDataList);
            this.flatListRef.current.setNativeProps({ scrollEnabled: true });
            this.dataWithProps = updatedDataList;
            this.resetClassVariables();
            const { disableAutoUpdate, updateListOnDragEnd } = this.props;
            if (!disableAutoUpdate && updateListOnDragEnd) {
                this.updateListOnDragEnd();
            }
        };
        this.onGestureEvent = event([
            {
                nativeEvent: ({ state, oldState, absoluteY }) => block([
                    cond(eq(this.isSavingValue, 0), [
                        set(this.gestureStateValue, state),
                        set(this.gestureOldStateValue, oldState),
                        onChange(this.gestureStateValue, [
                            cond(or(eq(state, GestureState.FAILED), eq(state, GestureState.END)), [
                                set(this.isSavingValue, 1),
                                call([this.activeItemIndexValue], this.handleSave),
                            ]),
                        ]),
                        cond(and(and(and(eq(this.initialSetUpFinishedValue, 1), eq(this.isSavingValue, 0)), eq(state, GestureState.ACTIVE)), greaterThan(this.activeItemIndexValue, -1)), [
                            cond(or(lessThan(absoluteY, this.containerOffsetTopValue), greaterThan(absoluteY, add(this.containerOffsetTopValue, this.containerHeightValue))), [
                                cond(lessThan(absoluteY, this.containerOffsetTopValue), [
                                    set(this.touchOutOfBoundsValue, 1),
                                    set(this.touchOutOfBoundsTopValue, 1),
                                ]),
                                cond(greaterThan(absoluteY, add(this.containerOffsetTopValue, this.containerHeightValue)), [
                                    set(this.touchOutOfBoundsValue, 1),
                                    set(this.touchOutOfBoundsBottomValue, 1),
                                ]),
                            ], [
                                set(this.touchOutOfBoundsValue, 0),
                                set(this.touchOutOfBoundsTopValue, 0),
                                set(this.touchOutOfBoundsBottomValue, 0),
                            ]),
                            cond(eq(this.touchOutOfBoundsValue, 0), [
                                set(this.pointerY, absoluteY),
                                call([
                                    this.pointerY,
                                    this.scrollTopValue,
                                    this.containerOffsetTopValue,
                                    this.activeItemArrayIndexValue,
                                ], this.trackHover),
                                cond(and(eq(this.hasAutoScrollStartedValue, 0), and(greaterThan(absoluteY, this.containerLowerBoundValue), lessThan(this.scrollTopValue, this.scrollTopLowerBoundValue))), [
                                    set(this.hasAutoScrollStartedValue, 1),
                                    set(this.isAutoScrollDownValue, 1),
                                    this.callAutoScroll,
                                ]),
                                cond(and(eq(this.hasAutoScrollStartedValue, 0), and(lessThan(absoluteY, this.containerUpperBoundValue), greaterThan(this.scrollTopValue, 0))), [
                                    set(this.hasAutoScrollStartedValue, 1),
                                    set(this.isAutoScrollUpValue, 1),
                                    this.callAutoScroll,
                                ]),
                                cond(and(eq(this.isAutoScrollDownValue, 1), lessThan(absoluteY, this.containerLowerBoundValue)), [
                                    set(this.hasAutoScrollStartedValue, 0),
                                    set(this.isAutoScrollDownValue, 0),
                                ]),
                                cond(and(eq(this.isAutoScrollUpValue, 1), greaterThan(absoluteY, this.containerUpperBoundValue)), [
                                    set(this.hasAutoScrollStartedValue, 0),
                                    set(this.isAutoScrollUpValue, 0),
                                ]),
                            ]),
                        ]),
                    ]),
                ]),
            },
        ]);
        this.runTiming = (clock, startingPosition, dest, idValue, arrayIndex, activeItemFinalTarget) => {
            const { activeItemIndexValue, activeItemArrayIndexValue, isSavingValue, } = this;
            const state = {
                finished: new Value(0),
                position: new Value(0),
                time: new Value(0),
                frameTime: new Value(0),
            };
            const config = {
                duration: 300,
                toValue: new Value(0),
                easing: Easing.inOut(Easing.ease),
            };
            return block([
                cond(eq(activeItemIndexValue, -1), [dest], cond(eq(idValue, activeItemIndexValue), [
                    cond(eq(isSavingValue, 1), [
                        set(startingPosition, activeItemFinalTarget),
                        activeItemFinalTarget,
                    ], dest),
                ], [
                    cond(neq(arrayIndex, activeItemArrayIndexValue), startingPosition, [
                        cond(clockRunning(clock), [set(config.toValue, dest)], cond(neq(state.position, dest), [
                            set(state.finished, 0),
                            set(state.time, 0),
                            set(state.position, startingPosition),
                            set(state.frameTime, 0),
                            set(config.toValue, dest),
                            startClock(clock),
                        ])),
                        timing(clock, state, config),
                        cond(state.finished, [stopClock(clock)]),
                        cond(eq(isSavingValue, 1), set(startingPosition, dest)),
                        state.position,
                    ]),
                ])),
            ]);
        };
        this.getItemTranslateY = (draggableId, arrayIndex, itemTranslateY) => {
            const { activeItemIndexValue, activeItemArrayIndexValue, activeHoverIndexValue, itemHeightValues, hiddenContentHeightValue, firstItemIds, lastItemIds, itemArrayIds, itemOffsets, pointerY, scrollTopValue, touchOutOfBoundsValue, touchOutOfBoundsTopValue, activeHoverOutOfBoundsValue, activeHoverZoneValue, containerOffsetTopValue, } = this;
            const clock = new Clock();
            const itemHeight = interpolate(activeItemArrayIndexValue, {
                inputRange: itemArrayIds,
                outputRange: itemHeightValues,
            });
            const itemOffset = itemOffsets[arrayIndex];
            const firstItemId = interpolate(activeItemArrayIndexValue, {
                inputRange: itemArrayIds,
                outputRange: firstItemIds,
            });
            const activeItemRelativeY = multiply(itemHeight, sub(draggableId, firstItemId));
            const activeItemOffset = add(itemOffset, activeItemRelativeY);
            const hoverBelowOffset = add(multiply(itemHeight, -1), itemTranslateY);
            const hoverAboveOffset = add(itemHeight, itemTranslateY);
            const outOfBoundTopPosition = multiply(sub(firstItemId, draggableId), itemHeight);
            const lastItemId = interpolate(this.activeItemArrayIndexValue, {
                inputRange: itemArrayIds,
                outputRange: lastItemIds,
            });
            const outOfBoundBottomPosition = multiply(sub(lastItemId, draggableId), itemHeight);
            const targetTranslateY = cond(eq(activeItemIndexValue, -1), itemTranslateY, [
                cond(eq(draggableId, activeItemIndexValue), add(cond(or(eq(touchOutOfBoundsValue, 1), eq(activeHoverOutOfBoundsValue, 1)), cond(or(eq(touchOutOfBoundsTopValue, 1), lessThan(activeHoverZoneValue, activeItemArrayIndexValue)), outOfBoundTopPosition, outOfBoundBottomPosition), add(sub(sub(sub(pointerY, divide(itemHeight, 2)), containerOffsetTopValue), activeItemOffset), multiply(divide(scrollTopValue, hiddenContentHeightValue), hiddenContentHeightValue))), itemTranslateY), cond(eq(activeHoverIndexValue, -1), itemTranslateY, cond(and(greaterThan(draggableId, activeItemIndexValue), lessOrEq(draggableId, activeHoverIndexValue)), hoverBelowOffset, cond(and(lessThan(draggableId, activeItemIndexValue), greaterOrEq(draggableId, activeHoverIndexValue)), hoverAboveOffset, itemTranslateY)))),
            ]);
            const clampedActiveHoverIndex = cond(lessThan(activeHoverIndexValue, firstItemId), firstItemId, cond(greaterThan(activeHoverIndexValue, lastItemId), lastItemId, activeHoverIndexValue));
            const activeItemFinalTarget = cond(eq(activeHoverIndexValue, -1), itemTranslateY, add(itemTranslateY, multiply(itemHeight, sub(clampedActiveHoverIndex, activeItemIndexValue))));
            const translateY = this.runTiming(clock, itemTranslateY, targetTranslateY, draggableId, arrayIndex, activeItemFinalTarget);
            return translateY;
        };
        this.draggableRenderItem = (renderProps) => {
            const { item: { draggableId, arrayIndex, itemTranslateY, draggableIdValue }, } = renderProps;
            const { activeItemIndexValue, activeHoverArrayIndexValue, firstItemRefs, firstItemIds, touchOutOfBoundsValue, activeHoverOutOfBoundsValue, headerRefs, } = this;
            const { isDataMultipleLists } = this.state;
            const hasMultipleRenderItems = Array.isArray(this.props.renderItem);
            const renderItem = hasMultipleRenderItems
                ? this.props.renderItem[arrayIndex]
                : this.props.renderItem;
            const renderHeader = Array.isArray(this.props.renderHeader)
                ? this.props.renderHeader[arrayIndex]
                : this.props.renderHeader;
            if (typeof renderItem !== 'function') {
                console.error('Missing "renderItem" prop: Either [1] mismatch of renderItem and data or [2] renderItem props is missing. [1] data is of object[][] and renderItem is not of fn[][]. [2] missing prop renderItem ');
            }
            const isListFirstItem = draggableId === 0 ||
                (isDataMultipleLists && firstItemIds.indexOf(draggableId) > -1);
            const itemRef = isListFirstItem ? firstItemRefs[arrayIndex] : null;
            const headerRef = isListFirstItem && headerRefs ? headerRefs[arrayIndex] : null;
            const opacity = cond(neq(draggableIdValue, activeItemIndexValue), 1, cond(or(and(neq(activeHoverArrayIndexValue, -1), neq(activeHoverArrayIndexValue, arrayIndex)), or(eq(activeHoverOutOfBoundsValue, 1), eq(touchOutOfBoundsValue, 1))), 0.5, 1));
            const translateY = this.getItemTranslateY(draggableIdValue, arrayIndex, itemTranslateY);
            const style = {
                transform: [{ translateY }],
                opacity,
            };
            return (<React.Fragment>
        {isListFirstItem && renderHeader ? (<Animated.View ref={headerRef}>{renderHeader()}</Animated.View>) : null}
        <Animated.View ref={itemRef} style={style}>
          {renderItem(renderProps)}
        </Animated.View>
      </React.Fragment>);
        };
    }
    componentDidUpdate(prevProps) {
        if (prevProps.data !== this.props.data && this.activeItemIndex === -1) {
            if (equals(this.props.data, this.dataToSave)) {
                return;
            }
            const isDataMultipleLists = getIsDataMultipleLists(this.props.data);
            const dataWithId = setUpInitialData(this.props.data);
            this.dataWithProps = dataWithId;
            this.dataToSave = this.props.data;
            this.setState({ dataWithId, isDataMultipleLists }, () => {
                this.handleFlatListLayout({
                    nativeEvent: {
                        layout: {
                            width: this.state.containerWidth,
                            height: this.state.containerHeight,
                        },
                    },
                });
            });
        }
    }
    render() {
        const { keyExtractor } = this.props;
        const { dataWithId, initialSetupFinished } = this.state;
        if (!dataWithId.length) {
            return null;
        }
        const enabled = initialSetupFinished === 1;
        return (<TapGestureHandler ref={this.tapRef} onHandlerStateChange={this.onTapStateChange} simultaneousHandlers={[this.longPressRef, this.panGestureRef]} maxDist={10000} enabled={enabled}>
        <Animated.View style={[styles.flexContainerOverflow]}>
          <LongPressGestureHandler ref={this.longPressRef} minDurationMs={300} onHandlerStateChange={this.onLongPress} simultaneousHandlers={[this.panGestureRef, this.tapRef]} maxDist={10000} enabled={enabled}>
            <Animated.View style={[styles.flexContainer]}>
              <PanGestureHandler ref={this.panGestureRef} onGestureEvent={this.onGestureEvent} onHandlerStateChange={this.onGestureEvent} simultaneousHandlers={[this.tapRef, this.longPressRef]} enabled={enabled}>
                <Animated.View style={[styles.flexContainer]}>
                  <AnimatedFlatList ref={this.flatListRef} onScroll={this.onScroll} onLayout={this.handleFlatListLayout} data={dataWithId} renderItem={this.draggableRenderItem} keyExtractor={keyExtractor} onContentSizeChange={this.setScrollLowerBound} removeClippedSubviews={false} scrollEventThrottle={16} scrollEnabled={enabled}/>
                </Animated.View>
              </PanGestureHandler>
            </Animated.View>
          </LongPressGestureHandler>
        </Animated.View>
      </TapGestureHandler>);
    }
}
SortableMultilist.defaultProps = {
    disableUpdateListDebounce: false,
    updateListDebounceDuration: 3000,
    disableAutoUpdate: false,
    enableRightDragArea: false,
    disableLongPress: false,
};
